=== modified file 'lib/lp/app/javascript/expander.js'
--- lib/lp/app/javascript/expander.js	2011-07-17 23:05:07 +0000
+++ lib/lp/app/javascript/expander.js	2011-07-18 09:23:10 +0000
@@ -50,6 +50,12 @@
  *         "expander" as its argument.  Once the loader has constructed the
  *         output Node or NodeList it wants to display ("output"), it calls
  *         expander.receive(output) to update the content node.
+ *     animate_node: A node to perform an animation on.  Mostly useful for
+ *         animating table rows/cells when you want to animate the inner
+ *         content (eg. a <div>).
+ *     no_animation: Set to true if no animation should be used.  Useful for
+ *         when you can't rearrange the nodes so animations apply to them
+ *         (eg. we want to show a bunch of rows in the same table).
  */
 function Expander(icon_node, content_node, config) {
     if (!Y.Lang.isObject(icon_node)) {
@@ -66,8 +72,19 @@
         this.config = {};
     }
     this.loaded = !Y.Lang.isValue(this.config.loader);
-    this._animation = Y.lazr.effects.reversible_slide_out(
-        this.content_node);
+
+    if (Y.Lang.isValue(this.config.animate_node)) {
+        this._animate_node = Y.one(this.config.animate_node);
+    } else {
+        this._animate_node = this.content_node;
+    }
+
+    if (this.config.no_animation !== true) {
+        this._animation = Y.lazr.effects.reversible_slide_out(
+            this._animate_node);
+    } else {
+        this._animation = undefined;
+    }
 
     // Is setup complete?  Skip any animations until it is.
     this.fully_set_up = false;
@@ -133,10 +150,12 @@
     foldContentNode: function(expand, no_animation) {
         var expander = this;
         var has_paused = false;
-        if (no_animation === true) {
+        if (no_animation === true || Y.Lang.isUndefined(this._animation)) {
             // Make the animation have the proper direction set from
             // the start.
-            this._animation.set('reverse', expand);
+            if (!Y.Lang.isUndefined(this._animation)) {
+                this._animation.set('reverse', expand);
+            }
             expander.setContentClassIf(
                 !expand, expander.css_classes.unseen);
         } else {
@@ -165,7 +184,7 @@
 
     revealIcon: function() {
         this.icon_node
-            .addClass('sprite').addClass('js-action')
+            .addClass('sprite')
             .removeClass('unseen');
     },
 
@@ -192,9 +211,12 @@
         if (failed === true) {
             this.loaded = false;
         }
-        var from_height = this.content_node.getStyle('height');
-        this.content_node.setContent(output);
-        var to_height = this.content_node.get('scrollHeight');
+        var from_height = this._animate_node.getStyle('height');
+        this._animate_node.setContent(output);
+        if (Y.Lang.isUndefined(this._animation)) {
+            return;
+        }
+        var to_height = this._animate_node.get('scrollHeight');
         if (this._animation.get('running')) {
             this._animation.stop();
         }
@@ -227,11 +249,28 @@
         this.setExpanded(expanded);
     },
 
+    /**
+     * Wrap node content in an <a> tag and mark it as js-action.
+     *
+     * @param node Y.Node object to modify: its content is modified
+     *     in-place so node events won't be lost, but anything set on
+     *     the inner content nodes might be.
+     */
+    wrapNodeWithLink: function(node) {
+        var link_node = Y.Node.create('<a></a>')
+            .addClass('js-action')
+            .set('href', '#')
+            .setContent(node.getContent());
+        node.setContent(link_node);
+    },
+
     /*
      * Set up an expander's DOM and event handler.
      *
+     * @param linkify {Boolean} Wrap the icon node into an <A> tag with
+     *     proper CSS classes and content from the icon node.
      */
-    setUp: function() {
+    setUp: function(linkify) {
         var expander = this;
         function click_handler(e) {
             e.halt();
@@ -239,6 +278,9 @@
         }
 
         this.render(this.isExpanded(), true);
+        if (linkify === true) {
+            this.wrapNodeWithLink(this.icon_node);
+        }
         this.icon_node.on('click', click_handler);
         this.revealIcon();
         this.fully_set_up = true;

=== modified file 'lib/lp/app/javascript/lp.js'
--- lib/lp/app/javascript/lp.js	2011-07-17 23:05:07 +0000
+++ lib/lp/app/javascript/lp.js	2011-07-18 09:23:10 +0000
@@ -85,37 +85,6 @@
     });
 }
 
-function toggleExpandableTableRow(element_id) {
-    var row = document.getElementById(element_id);
-    var view_icon = document.getElementById(element_id + "-arrow");
-    if (row.style.display == "table-row") {
-      row.style.display = "none";
-      view_icon.setAttribute("src", "/@@/treeCollapsed");
-    } else {
-      row.style.display = "table-row";
-      view_icon.setAttribute("src", "/@@/treeExpanded");
-    }
-    return false;
-}
-
-function toggleExpandableTableRows(class_name) {
-    var view_icon = document.getElementById(class_name + "-arrow");
-    var all_page_tags = document.getElementsByTagName("*");
-    for (i = 0; i < all_page_tags.length; i++) {
-        row = all_page_tags[i];
-        if (row.className == class_name) {
-            if (row.style.display == "table-row") {
-              row.style.display = "none";
-              view_icon.setAttribute("src", "/@@/treeCollapsed");
-            } else {
-              row.style.display = "table-row";
-              view_icon.setAttribute("src", "/@@/treeExpanded");
-            }
-        }
-    }
-    return false;
-}
-
 // Enable or disable the beta.launchpad.net redirect
 function setBetaRedirect(enable) {
     var expire = new Date();

=== modified file 'lib/lp/app/javascript/tests/test_expander.js'
--- lib/lp/app/javascript/tests/test_expander.js	2011-07-17 23:05:07 +0000
+++ lib/lp/app/javascript/tests/test_expander.js	2011-07-18 09:23:10 +0000
@@ -85,6 +85,24 @@
             return expander;
         },
 
+        test_separate_animate_node: function() {
+            var icon = Y.Node.create('<td></td>'),
+                content = Y.Node.create('<td></td>'),
+                animate = Y.Node.create('<div></div>');
+            var expander = new module.Expander(icon, content,
+                                               { animate_node: animate });
+            Y.Assert.areSame(content, expander.content_node);
+            Y.Assert.areSame(animate, expander._animation.get('node'));
+        },
+
+        test_no_animate_node: function() {
+            // When config.no_animation is true, no animation
+            // is constructed or used.
+            var expander = this.makeExpander(
+                undefined, { config: { no_animation: true } });
+            Y.Assert.isUndefined(expander._animation);
+        },
+
         test_loaded_is_true_if_no_loader_is_defined: function() {
             var icon = Y.Node.create('<p></p>'),
                 content = Y.Node.create('<p></p>');
@@ -120,7 +138,6 @@
             icon.addClass('unseen');
             var expander = this.makeExpander(root);
             Y.Assert.isFalse(icon.hasClass('unseen'));
-            Y.Assert.isTrue(icon.hasClass('js-action'));
         },
 
         test_setUp_hides_content_by_default: function() {
@@ -173,6 +190,21 @@
             Y.Assert.isTrue(render_has_run);
         },
 
+        test_setUp_linkifies_when_asked: function() {
+            var wrap_has_run = false;
+            var fake_wrapNodeWithLink = function() {
+                wrap_has_run = true;
+            };
+
+            root = this.makeExpanderHooks();
+            var expander = new module.Expander(
+                root.one('.icon'), root.one('.content'));
+            expander.wrapNodeWithLink = fake_wrapNodeWithLink;
+
+            expander.setUp(true);
+            Y.Assert.isTrue(wrap_has_run);
+        },
+
         test_setUp_calls_foldContentNode_no_anim: function() {
             var foldContentNode_animate_arg = false;
             var fake_foldContentNode = function(expanded, no_animate) {
@@ -261,7 +293,7 @@
             var content_node = Y.Node.create('<div />')
                 .setStyle('height', '2px');
             this.findTestHookTag().appendChild(content_node);
-            expander.content_node = content_node;
+            expander.content_node = expander._animate_node = content_node;
 
             // Full desired content height of 5px.
             var content = Y.Node.create('<div />')
@@ -375,6 +407,21 @@
             // in progress instead.
             anim.fire('end');
             Y.Assert.isFalse(expander.content_node.hasClass("unseen"));
+        },
+
+        test_wrapNodeWithLink: function() {
+            // Wraps node content with an <a href="#" class="js-action"> tag.
+            var node = Y.Node.create('<div></div>')
+                .set('text', 'Test');
+            var expander = this.makeExpander();
+            expander.wrapNodeWithLink(node);
+            var link_node = node.one('a');
+            Y.Assert.isNotNull(link_node);
+            Y.Assert.areSame('Test', link_node.get('text'));
+            Y.Assert.isTrue(link_node.hasClass('js-action'));
+            // Link href is '#', but we get full test path appended
+            // so instead we just ensure we've got a string back.
+            Y.Assert.isString(link_node.get('href'));
         }
     }));
 

=== modified file 'lib/lp/bugs/javascript/filebug_dupefinder.js'
--- lib/lp/bugs/javascript/filebug_dupefinder.js	2011-07-17 23:05:07 +0000
+++ lib/lp/bugs/javascript/filebug_dupefinder.js	2011-07-18 09:23:10 +0000
@@ -77,19 +77,6 @@
 }
 
 /**
- * Collapse the details for a bug and set its expander arrow to
- * 'collapsed'
- * @param expander The expander to collapse.
- */
-function collapse_bug_details(expander) {
-    var bug_details_div = get_details_div(expander);
-    var anim = Y.lazr.effects.slide_in(bug_details_div);
-    anim.run();
-
-    expander.set(SRC, EXPANDER_COLLAPSED);
-}
-
-/**
  * Show the bug reporting form and collapse all bug details forms.
  */
 function show_bug_reporting_form() {
@@ -198,7 +185,7 @@
         duplicate_div.set(INNER_HTML, response.responseText);
 
         bug_already_reported_expanders = Y.all(
-            'img.bug-already-reported-expander');
+            'td.bug-already-reported-expander');
         if (bug_already_reported_expanders.size() > 0) {
             // If there are duplicates shown, set up the JavaScript of
             // the duplicates that have been returned.
@@ -370,31 +357,19 @@
 
 namespace.setup_dupes = function() {
     bug_already_reported_expanders = Y.all(
-        'img.bug-already-reported-expander');
+        'td.bug-already-reported-expander');
     var bug_reporting_form = Y.one('#bug-reporting-form');
 
     if (bug_already_reported_expanders.size() > 0) {
-        // Collapse all the details divs, since we don't want them
-        // expanded first up.
-        Y.each(Y.all('div.duplicate-details'), function(div) {
-            collapse_bug_details(div);
-        });
-
         // Set up the onclick handlers for the expanders.
         Y.each(Y.all('.similar-bug'), function(row) {
             var bug_details_div = row.one('div.duplicate-details');
-            var image = row.one('img.bug-already-reported-expander');
             var bug_title_link = row.one('.duplicate-bug-link');
+            bug_title_link.addClass('js-action');
             var view_bug_link = row.one('.view-bug-link');
-
-            // Shut down the default action for the link and mark it
-            // as a JS'd link. We do this as it's simpler than
-            // trying to find all the bits of the row that we want
-            // to make clickable.
-            bug_title_link.addClass('js-action');
-            bug_title_link.on('click', function(e) {
-                e.preventDefault();
-            });
+            var expander = new Y.lp.app.widgets.expander.Expander(
+                row, bug_details_div);
+            expander.setUp();
 
             // The "view this bug" link shouldn't trigger the
             // collapsible, so we stop the event from propagating.
@@ -410,14 +385,25 @@
                 e.stopPropagation();
             });
 
+            // Preventing default behaviour stops the browser from
+            // emitting the focus event here as well.
+            bug_title_link.on('click', function(e) {
+                // Stop the click event from being emitted down
+                // to the container node.
+                if (this.focused_expansion === true &&
+                    expander._animation.get('running')) {
+                    e.stopPropagation();
+                }
+                this.focused_expansion = false;
+                e.preventDefault();
+            });
+
             // Set up the on focus handler for the link so that
             // tabbing will expand the different bugs.
             bug_title_link.on('focus', function(e) {
-                if (!bug_details_div.hasClass('lazr-opened')) {
-                    var anim = Y.lazr.effects.slide_out(bug_details_div);
-                    anim.run();
-
-                    image.set(SRC, EXPANDER_EXPANDED);
+                if (!expander.isExpanded()) {
+                    this.focused_expansion = true;
+                    expander.render(true);
 
                     // If the bug reporting form is shown, hide it.
                     if (bug_reporting_form.getStyle(DISPLAY) == BLOCK) {
@@ -425,17 +411,6 @@
                     }
                 }
             });
-
-            row.on('click', function(e) {
-                if (bug_details_div.hasClass('lazr-opened')) {
-                    collapse_bug_details(image);
-                } else {
-                    var anim = Y.lazr.effects.slide_out(bug_details_div);
-                    anim.run();
-
-                    image.set(SRC, EXPANDER_EXPANDED);
-                }
-            });
         });
 
         // Hide the bug reporting form.
@@ -503,4 +478,4 @@
 
 }, "0.1", {"requires": [
     "base", "io", "oop", "node", "event", "json", "lazr.formoverlay",
-    "lazr.effects"]});
+    "lazr.effects", "lp.app.widgets.expander"]});

=== modified file 'lib/lp/bugs/javascript/tests/test_filebug_dupfinder.js'
--- lib/lp/bugs/javascript/tests/test_filebug_dupfinder.js	2011-07-17 23:05:07 +0000
+++ lib/lp/bugs/javascript/tests/test_filebug_dupfinder.js	2011-07-18 09:23:10 +0000
@@ -142,12 +142,11 @@
         search_text.set('value', 'foo');
         var search_button = Y.one(Y.DOM.byId('field.actions.search'));
         this.config.yio.io.responseText = ([
-                '<img id="bug-details-expander" ',
-                'class="bug-already-reported-expander" ',
-                'src="/@@/treeCollapsed">',
+                '<table><tr><td id="bug-details-expander" ',
+                'class="bug-already-reported-expander"></td></tr></table>',
                 '<input type="button" value="No, I need to report a new bug"',
                 ' name="field.bug_already_reported_as"',
-                ' id="bug-not-already-reported" style="display: block">'
+                ' id="bug-not-already-reported" style="display: block">',
                 ].join(''));
         this.config.yio.io.doAfter = function() {
             // filebug container should not be visible when there are dups

=== modified file 'lib/lp/bugs/templates/bugtarget-macros-filebug.pt'
--- lib/lp/bugs/templates/bugtarget-macros-filebug.pt	2011-07-17 23:05:07 +0000
+++ lib/lp/bugs/templates/bugtarget-macros-filebug.pt	2011-07-18 09:23:10 +0000
@@ -346,13 +346,7 @@
       <table tal:define="bugtask python:view.getRelevantBugTask(bug)">
         <tbody>
           <tr>
-            <td>
-              <img width="14" height="14" src="/@@/treeCollapsed"
-                  class="bug-already-reported-expander"
-                  tal:attributes="id string:bug-details-expander-bug-${bug/id};"
-                />
-            </td>
-            <td>
+            <td class="bug-already-reported-expander">
               <label tal:attributes="for string:bug-already-reported-as-${bug/id};
                                      class bugtask/image:sprite_css"
                      tal:condition="bugtask"/>

=== modified file 'lib/lp/bugs/templates/bugtask-tasks-and-nominations-table-row.pt'
--- lib/lp/bugs/templates/bugtask-tasks-and-nominations-table-row.pt	2011-07-17 23:05:07 +0000
+++ lib/lp/bugs/templates/bugtask-tasks-and-nominations-table-row.pt	2011-07-18 09:23:10 +0000
@@ -9,17 +9,14 @@
           form_row_id string:task${context/id}">
   <tr tal:define="editstatus_url string:${context/fmt:url}/+editstatus"
       tal:attributes="class view/getTaskRowCSSClass; id row_id">
-    <td class="icon left right">
+    <td>
       <metal:expander
           metal:define-macro="expander"
           tal:define="expander_link_text expander_link_text|nothing">
         <a tal:condition="expandable"
-           tal:attributes="
-            href tasklink;
-            onclick string:return toggleFormVisibility('${form_row_id}')"
-            class="sprite bug-status-expand">
+           tal:attributes="href tasklink" class="bugtask-expander">
+          <tal:link_text tal:replace="expander_link_text" />
           &nbsp;
-          <tal:link_text tal:replace="expander_link_text" />
         </a>
         <tal:no_expander tal:condition="not: expandable"
                          tal:replace="expander_link_text" />
@@ -175,19 +172,27 @@
           class="bugtasks-table-row-init-script"
           tal:condition="not:view/many_bugtasks"
           tal:content="string:
-    LPS.use('event', 'lp.bugs.bugtask_index', function(Y) {
-        Y.on('load',
-             function(e) {
-                 Y.lp.bugs.bugtask_index.setup_bugtask_row(${view/js_config});
-             },
-             window);
+    LPS.use('event', 'lp.bugs.bugtask_index', 'lp.app.widgets.expander',
+            function(Y) {
+        Y.on('domready', function() {
+          Y.lp.bugs.bugtask_index.setup_bugtask_row(${view/js_config});
+
+          // Set-up the expander on the bug task summary row.
+          var icon_node = Y.one('tr#${row_id} a.bugtask-expander');
+          var row_node = Y.one('tr#${form_row_id}');
+          var content_node = row_node.one('td form');
+          var expander = new Y.lp.app.widgets.expander.Expander(
+            icon_node, row_node, { animate_node: content_node });
+          expander.setUp();
+        });
     });
   "/>
+
   <tal:form condition="view/displayEditForm">
     <tr
       tal:attributes="id form_row_id"
       tal:condition="expandable"
-      style="display: none"
+      class="bugtask-collapsible-content unseen"
     >
      <td colspan="7" tal:content="structure view/edit_view" />
     </tr>

=== modified file 'lib/lp/bugs/templates/bugtasks-and-nominations-table.pt'
--- lib/lp/bugs/templates/bugtasks-and-nominations-table.pt	2011-07-17 23:05:07 +0000
+++ lib/lp/bugs/templates/bugtasks-and-nominations-table.pt	2011-07-18 09:23:10 +0000
@@ -59,19 +59,6 @@
   </tal:not-editable>
 </tal:affects-me-too>
 
-<script type="text/javascript">
-  function toggleFormVisibility(row_id) {
-    row = document.getElementById(row_id)
-    edit_icon = document.getElementById(row_id + "-edit")
-    if (row.style.display=="none") {
-      row.style.display = "table-row";
-    } else {
-      row.style.display = "none";
-    }
-    return false;
-  }
-</script>
-
 <table
   id="affected-software"
   tal:attributes="class python: context.duplicateof and 'duplicate listing' or 'listing'"

=== modified file 'lib/lp/soyuz/browser/publishing.py'
--- lib/lp/soyuz/browser/publishing.py	2011-07-17 23:05:07 +0000
+++ lib/lp/soyuz/browser/publishing.py	2011-07-18 09:23:10 +0000
@@ -172,23 +172,6 @@
         return self.context.dateremoved is not None
 
     @property
-    def js_connector(self):
-        """Return the javascript glue for expandable rows mechanism."""
-        return """
-        <script type="text/javascript">
-           registerLaunchpadFunction(function() {
-               // Set the style of the expander icon so that it appears
-               // clickable when js is enabled:
-               var view_icon = document.getElementById('pub%s-expander');
-               view_icon.style.cursor = 'pointer';
-               connect('pub%s-expander', 'onclick', function (e) {
-                   toggleExpandableTableRow('pub%s');
-                   });
-               });
-        </script>
-        """ % (self.context.id, self.context.id, self.context.id)
-
-    @property
     def removal_comment(self):
         """Return the removal comment or 'None provided'."""
         removal_comment = self.context.removal_comment

=== modified file 'lib/lp/soyuz/stories/soyuz/xx-queue-pages.txt'
--- lib/lp/soyuz/stories/soyuz/xx-queue-pages.txt	2011-07-17 23:05:07 +0000
+++ lib/lp/soyuz/stories/soyuz/xx-queue-pages.txt	2011-07-18 09:23:10 +0000
@@ -205,15 +205,15 @@
   >>> anon_browser.getControl(name="queue_text").value = ''
   >>> anon_browser.getControl("Update").click()
 
-  >>> print find_tag_by_id(anon_browser.contents, 'queue-4-arrow')
-  <img width="14" height="14" src="/@@/treeCollapsed" alt="view files"
-       id="queue-4-arrow" />
+  >>> print find_tag_by_id(anon_browser.contents, 'queue-4-icon')
+  <span class="expander-link" id="queue-4-icon">&nbsp;</span>
 
 The 'filelist' is expanded as one or more table rows, right below the
 clicked item:
 
-  >>> filelist = find_tags_by_class(
+  >>> filelist_body = first_tag_by_class(
   ...     anon_browser.contents, 'queue-4')
+  >>> filelist = filelist_body.findAll('tr')
 
 It contains a list of files related to the queue item clicked, followed
 by its size, one file per line. Expired files have no size.

=== modified file 'lib/lp/soyuz/templates/archive-index.pt'
--- lib/lp/soyuz/templates/archive-index.pt	2011-07-17 23:05:07 +0000
+++ lib/lp/soyuz/templates/archive-index.pt	2011-07-18 09:23:10 +0000
@@ -95,10 +95,8 @@
               target="help">Read about installing</a>)
         </p>
 
-        <div id="ppa-install-slide-trigger">
-          <div class="widget-header">
-            Technical details about this PPA
-          </div>
+        <div class="widget-header">
+          Technical details about this PPA
         </div>
 
         <div class="widget-body">
@@ -148,34 +146,13 @@
             >Celso Providelo</a>.</p>
 
         <script type="text/javascript">
-    LPS.use('node', 'event', 'lazr.effects', function(Y) {
-
-    // Hide the widget body contents.
-    Y.one('#ppa-install .widget-body').addClass('lazr-closed');
-
-    // Ensure that the widget header uses the correct sprite icon
-    // and gets the styling for javascript actions applied.
-    var widget_header = Y.one('#ppa-install .widget-header');
-    widget_header.addClass('sprite');
-    widget_header.addClass('treeCollapsed');
-    widget_header.addClass('js-action');
-
-    var slide;
-    function toggle_body_visibility(e) {
-        if (!slide) {
-            slide = Y.lazr.effects.slide_out('#ppa-install .widget-body');
-            widget_header.replaceClass('treeCollapsed', 'treeExpanded');
-        } else {
-            slide.set('reverse', !slide.get('reverse'));
-            widget_header.toggleClass('treeExpanded');
-            widget_header.toggleClass('treeCollapsed');
-        }
-        slide.stop();
-        slide.run();
-    }
-    Y.on('click', toggle_body_visibility,
-         '#ppa-install-slide-trigger');
-});
+          LPS.use('lp.app.widgets.expander', function(Y) {
+              var widget_header = Y.one('#ppa-install .widget-header');
+              var content = Y.one('.widget-body');
+              var expander = new Y.lp.app.widgets.expander.Expander(
+                  widget_header, content);
+              expander.setUp(true);
+          });
         </script>
       </tal:is-active>
 

=== modified file 'lib/lp/soyuz/templates/archive-macros.pt'
--- lib/lp/soyuz/templates/archive-macros.pt	2011-07-17 23:05:07 +0000
+++ lib/lp/soyuz/templates/archive-macros.pt	2011-07-18 09:23:10 +0000
@@ -10,7 +10,8 @@
   </tal:comment>
 
 <script type="text/javascript">
-LPS.use('node', 'io-base', 'lazr.anim', 'lp.soyuz.base', function(Y) {
+LPS.use('node', 'io-base', 'lazr.anim', 'lp.soyuz.base',
+        'lp.app.widgets.expander', function(Y) {
 
 
 /*
@@ -18,19 +19,13 @@
  * the operation.
  */
 function informFailure(transaction_id, response, args) {
-    function retry_handler(e) {
-        e.preventDefault();
-        startUpdate(args.container);
-    };
-
     var failure_message = Y.lp.soyuz.base.makeFailureNode(
-        'Failed to fetch package details.', retry_handler);
+        'Failed to fetch package details.');
 
-    args.container.set('innerHTML', '');
-    args.container.appendChild(failure_message);
+    args.expander.receive(failure_message);
 
     var anim = Y.lazr.anim.red_flash({
-        node: args.container
+        node: args.expander.content_node
         });
     anim.run();
 }
@@ -40,19 +35,16 @@
  * Update the row with the XHR response.
  */
 function doUpdate(transaction_id, response, args) {
-    args.container.set('innerHTML', response.responseText);
+    var node = Y.Node.create('<div />')
+                 .set('innerHTML', response.responseText);
+    args.expander.receive(node);
 }
 
 
-/*
- * Dispatch a XHR for updating the given container.
+/**
+ * Dispatch a XHR to load the given container.
  */
-function startUpdate(container) {
-    var in_progress_message = Y.lp.soyuz.base.makeInProgressNode(
-        'Fetching package details ...')
-
-    container.set('innerHTML', '');
-    container.appendChild(in_progress_message);
+function loadDetails(expander) {
 
     var config = {
         on: {
@@ -60,11 +52,11 @@
             'failure': informFailure
         },
         arguments: {
-            'container': container,
+            'expander': expander
         }
     };
 
-    /*
+    /**
      * If a page wants to use this outside of an archive context then it
      * can define LP.cache['archive_context'], which should be a
      * full or relative URL to the context archive page required.
@@ -74,63 +66,37 @@
             ? LP.cache['archive_context_url']
             : '';
     if (base_url !== '') {
-        base_url += '/'
+        base_url += '/';
     }
-    var pub_id = container.get('id').replace(
-        '-container', '').replace('pub', '');
+    var pub_id = expander.content_node.get('id')
+                   .replace('pub', '')
+                   .replace('-container', '');
     var uri = base_url + '+sourcepub/' + pub_id + '/+listing-archive-extra';
     Y.io(uri, config);
 }
 
-
-/*
- * Toggle the visibility of the expander targeted and the visual of
- * the expander itself.
- */
-function toggleExpandableRow(expander) {
-    var row = Y.one('#' + expander.get('id').replace('-expander', ''));
-    var icon = expander.one('img');
-
-    var row_display = row.getStyle('display');
-    if (row_display == 'none') {
-        row.setStyle('display', 'table-row');
-        icon.set('src', '/@@/treeExpanded');
-        var container = Y.one('#' + row.get('id') + '-container');
-        if (trim(container.get('innerHTML')) == ''){
-            startUpdate(container);
-        }
-    }
-    else {
-        row.setStyle('display', 'none');
-        icon.set('src', '/@@/treeCollapsed');
-    }
-}
-
-
-/*
- * Setup expander handlers and pre-load 'treeExpanded', 'no' and
- * 'spinner' images for better reponsiveness.
+/**
+ * Setup expander handlers.
  */
 function setupPackageListExpanders() {
-    var spinner = new Image();
-    spinner.src = '/@@/spinner';
-
-    var no = new Image();
-    no.src = '/@@/no';
-
-    var tree_expanded = new Image();
-    tree_expanded.src = '/@@/treeExpanded';
-
-    function expander_handler (e) {
-        e.preventDefault();
-        toggleExpandableRow(e.currentTarget);
-    };
-
-    // Note: there are situations, such as displaying empty result
-    // sets, when there will not be any links for expanders on the page.
     var expanders = Y.all('#packages_list a.expander');
-    if (expanders !== null){
-        expanders.on("click", expander_handler);
+    var widget;
+    if (expanders !== null) {
+       function setupExpander(expander) {
+         var base_id = expander.get('id').replace('-expander', '');
+         var container = Y.one('#' + base_id);
+         var content = container.one('td div.package-details');
+         var in_progress_message = Y.lp.soyuz.base.makeInProgressNode(
+           'Fetching package details ...');
+         content.empty();
+         content.appendChild(in_progress_message);
+
+         widget = new Y.lp.app.widgets.expander.Expander(
+             expander, container, { loader: loadDetails,
+                                    animate_node: content });
+         widget.setUp();
+       };
+       expanders.each(setupExpander);
     }
 }
 

=== modified file 'lib/lp/soyuz/templates/distributionsourcepackage-publishinghistory.pt'
--- lib/lp/soyuz/templates/distributionsourcepackage-publishinghistory.pt	2011-07-17 23:05:07 +0000
+++ lib/lp/soyuz/templates/distributionsourcepackage-publishinghistory.pt	2011-07-18 09:23:10 +0000
@@ -9,6 +9,24 @@
 
 <body>
 
+<div metal:fill-slot="head_epilogue">
+  <script type="text/javascript">
+    LPS.use('node', 'lp.app.widgets.expander', function(Y) {
+      Y.on('domready', function() {
+        var all_expanders = Y.all('.expander-icon');
+        all_expanders.each(function(icon) {
+          var base_id = icon.get('id').replace('-expander', '');
+          var content_node = Y.one('#' + base_id);
+          var animate_node = content_node.one('ul');
+          var expander = new Y.lp.app.widgets.expander.Expander(
+            icon, content_node, { animate_node: animate_node });
+          expander.setUp();
+        });
+      });
+    });
+  </script>
+</div>
+
 <div metal:fill-slot="main">
 
   <div class="top-portlet">

=== modified file 'lib/lp/soyuz/templates/distroseries-queue.pt'
--- lib/lp/soyuz/templates/distroseries-queue.pt	2011-07-17 23:05:07 +0000
+++ lib/lp/soyuz/templates/distroseries-queue.pt	2011-07-18 09:23:10 +0000
@@ -9,6 +9,24 @@
 
 <body>
 
+<div metal:fill-slot="head_epilogue">
+  <script type="text/javascript">
+    LPS.use('node', 'lp.app.widgets.expander', function(Y) {
+        Y.on('domready', function() {
+            var all_expanders = Y.all('.expander-link');
+            all_expanders.each(function(link) {
+                var base_id = link.get('id').replace('-icon', '');
+                var content = Y.one('tbody.' + base_id);
+                var content_rows = content.all('tr td *');
+                var expander = new Y.lp.app.widgets.expander.Expander(
+                    link, content, { no_animation: true });
+                expander.setUp();
+            });
+        });
+    });
+  </script>
+</div>
+
 <div metal:fill-slot="main"
      tal:define="setup view/setupQueueList;
                  message view/performQueueAction;
@@ -68,9 +86,9 @@
            <th> When </th>
          </tr>
        </thead>
-       <tbody class="lesser">
-         <tal:batch repeat="packageupload batch">
-           <tal:block define="filelist_class string:queue-${packageupload/id}">
+       <tal:batch repeat="packageupload batch">
+         <tal:block define="filelist_class string:queue-${packageupload/id}">
+           <tbody class="lesser">
              <tr class="queue-row">
                 <tal:comment condition="nothing">
                   Every column is top-padded apart from the checkbox
@@ -78,12 +96,8 @@
                   other browsers look odd, now. :/
                 </tal:comment>
                 <td width="14" style="padding-top: 5px">
-                  <a tal:attributes="href string:#;
-                     onclick string:return toggleExpandableTableRows('${filelist_class}')">
-                    <img tal:attributes="id string:${filelist_class}-arrow"
-                         width="14" height="14"
-                         src="/@@/treeCollapsed" alt="view files"/>
-                  </a>
+                  <span tal:attributes="id string:${filelist_class}-icon"
+                        class="expander-link">&nbsp;</span>
                 </td>
                 <td class="icon left" tal:condition="view/availableActions">
                      <input type="checkbox" name="QUEUE_ID"
@@ -128,9 +142,13 @@
                   </span>
                 </td>
              </tr>
+           </tbody>
+           <tbody tal:attributes="class string:${filelist_class} lesser">
              <metal:filelist use-macro="template/macros/package-filelist"/>
-           </tal:block>
-         </tal:batch>
+           </tbody>
+         </tal:block>
+       </tal:batch>
+       <tbody class="lesser">
          <tr tal:condition="view/availableActions">
            <td colspan="4" style="text-align: right;
                                   vertical-align: bottom;
@@ -203,10 +221,8 @@
   </tal:comment>
 
     <tal:copy condition="packageupload/pending_delayed_copy">
-      <tr tal:attributes="class string:${filelist_class}"
-          tal:define="archive
-                      packageupload/sourcepackagerelease/upload_archive"
-          style="display:none">
+      <tr tal:define="archive
+                      packageupload/sourcepackagerelease/upload_archive">
         <td />
         <td tal:condition="view/availableActions" />
         <td>Copied from
@@ -224,9 +240,7 @@
     </tal:copy>
 
     <tal:upload condition="not: packageupload/pending_delayed_copy">
-      <tr tal:repeat="file packageupload/source_files"
-          tal:attributes="class string:${filelist_class}"
-          style="display:none">
+      <tr tal:repeat="file packageupload/source_files">
         <td/>
         <td tal:condition="view/availableActions"/>
         <td
@@ -243,9 +257,7 @@
                 component_name package/component/name/lower;
                 section_name package/section/name/lower;
                 version package/version">
-        <tr tal:repeat="file python:packageupload.binary_packages[package]"
-            tal:attributes="class string:${filelist_class}"
-            style="display:none">
+        <tr tal:repeat="file python:packageupload.binary_packages[package]">
           <td/>
           <td tal:condition="view/availableActions"/>
           <td tal:define="libraryfilealias file/libraryfile">
@@ -262,9 +274,7 @@
     </tal:package>
 
     <tal:custom define="customfiles packageupload/customfiles">
-      <tr tal:repeat="custom python:list(customfiles)"
-          tal:attributes="class string:${filelist_class}"
-          style="display:none">
+      <tr tal:repeat="custom python:list(customfiles)">
         <td/>
         <td tal:condition="view/availableActions"></td>
         <td tal:define="libraryfilealias custom/libraryfilealias">
@@ -276,9 +286,7 @@
 
     <tal:diffs condition="packageupload/sourcepackagerelease">
       <tr tal:define="diffs packageupload/sourcepackagerelease/package_diffs"
-          tal:repeat="diff diffs"
-          tal:attributes="class string:${filelist_class}"
-          style="display:none">
+          tal:repeat="diff diffs">
         <td/>
         <td tal:condition="view/availableActions"></td>
         <td>

=== modified file 'lib/lp/soyuz/templates/publishinghistory-macros.pt'
--- lib/lp/soyuz/templates/publishinghistory-macros.pt	2011-07-17 23:05:07 +0000
+++ lib/lp/soyuz/templates/publishinghistory-macros.pt	2011-07-18 09:23:10 +0000
@@ -10,10 +10,9 @@
       :version_url: The version link
   </tal:comment>
   <tr>
-    <td style="white-space: nowrap"><span
-        tal:attributes="id string:pub${context/id}-expander"
-        ><img src="/@@/treeCollapsed" alt=""
-                tal:attributes="id string:pub${context/id}-arrow" /></span>
+    <td style="white-space: nowrap">
+      <span tal:attributes="id string:pub${context/id}-expander"
+            class="expander-icon">&nbsp;</span>
     </td>
     <td tal:content="view/date_last_changed/fmt:datetime"
         >2005-08-24</td>
@@ -31,11 +30,11 @@
       >2.0.39</a>
     </td>
   </tr>
-  <tr tal:attributes="id string:pub${context/id}" style="display: none">
+  <tr tal:attributes="id string:pub${context/id}">
     <td colspan="8">
       <tal:block replace="structure context/@@+record-details" />
     </td>
   </tr>
-  <tal:js_glue replace="structure view/js_connector" />
+
 </metal:header>
 

=== modified file 'lib/lp/soyuz/templates/sourcepackagepublishinghistory-listing-archive-detailed.pt'
--- lib/lp/soyuz/templates/sourcepackagepublishinghistory-listing-archive-detailed.pt	2011-07-17 23:05:07 +0000
+++ lib/lp/soyuz/templates/sourcepackagepublishinghistory-listing-archive-detailed.pt	2011-07-18 09:23:10 +0000
@@ -13,11 +13,10 @@
         tal:condition="view/allow_selection"
         tal:attributes="value context/id;
                         id string:field.selected_sources.${context/id}"/>
-      <a class="expander"
-         tal:attributes="id string:pub${context/id}-expander;
-                         href string:+sourcepub/${context/id}/+listing-archive-extra">
-        <img src="/@@/treeCollapsed" alt="Show details" title="Show details"
-             tal:attributes="id string:pub${context/id}-arrow" />
+      <a tal:attributes="
+           href string:+sourcepub/${context/id}/+listing-archive-extra;
+           id string:pub${context/id}-expander"
+         class="sprite expander">
         <img src="/@@/package-source" />
         <tal:source_name replace="context/sourcepackagerelease/title">
           foo - 1.0
@@ -69,8 +68,8 @@
       </tal:related_builds>
     </td>
   </tr>
-  <tr tal:attributes="id string:pub${context/id}" style="display: none">
-    <td colspan="7">
+  <tr tal:attributes="id string:pub${context/id}">
+    <td colspan="7" style="padding: 0px;">
       <div class="package-details"
            tal:attributes="id string:pub${context/id}-container" />
     </td>

